import logging
from datetime import datetime

from pydantic import BaseModel, Field
from sqly import Dialect, Query

logger = logging.getLogger(__name__)


class Job(BaseModel):
    qname: str = Field()
    id: int = Field(default=None)
    retries: int = Field(default=3)
    queued: datetime = Field(default=None)
    scheduled: datetime = Field(default=None)
    data: dict = Field(default_factory=dict)


class Queue(BaseModel):
    qname: str
    dialect: Dialect

    def __post_init__(self):
        logger.info('Queue initialized: %r' % self)

    def put(self, data, retries=3, scheduled=None):
        job = Job(qname=self.qname, data=data, retries=retries, scheduled=scheduled)
        job_data = job.dict(exclude_none=True)
        return self.dialect.render(
            f"""
            INSERT INTO qy_jobs ({Query.fields(job_data)}) 
            VALUES ({Query.params(job_data)})
            RETURNING *
            """,
            job_data,
        )

    def get(self):
        return self.dialect.render(
            """
            UPDATE qy_jobs q1 SET retries = retries - 1
            WHERE q1.id = ( 
                SELECT q2.id FROM qy_jobs q2 
                WHERE q2.qname=:qname
                AND q2.retries > 0
                AND q2.scheduled <= now()
                ORDER BY q2.queued 
                FOR UPDATE SKIP LOCKED LIMIT 1 
            )
            RETURNING q1.*;
            """,
            {'qname': self.qname}
        )

    def delete(self, job):
        return self.dialect.render(
            """
            DELETE FROM qy_jobs WHERE id=:id
            """,
            {'id': job.id},
        )



import os, json, re
from setuptools import setup, find_packages
from codecs import open

PATH = os.path.dirname(os.path.abspath(__file__))

with open(os.path.join(PATH, "setup.json"), encoding='utf-8') as f:
    CONFIG = json.load(f)

with open(os.path.join(PATH, 'README.md'), encoding='utf-8') as f:
    README = f.read()


def req_from_file(filename):
    """
    Return a list of requirements, suitable for setup.py install, from a pip -r req.txt file
    """
    txt = open(filename, 'rb').read().decode('utf-8')
    lines = re.split(r'[\n\r]+', txt)
    return [line.split('#')[0].strip() for line in lines if line and line[0:2] != '-r']


if __name__ == '__main__':
    setup(
        long_description=README,
        long_description_content_type='text/markdown',
        packages=find_packages(),
        include_package_data=True,
        # install_requires=req_from_file(os.path.join(PATH, 'req/install.txt')),
        **CONFIG
    )
